#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdio.h>
#include <ctype.h>
#include <error.h>
#include <errno.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>

#include "client-serveur.h"

int main(int argc, char *argv[]) {
  message_t msgConnexion, msgRepConnexion, msgRequete, msgRepRequete;
  int fdW, fdR;
  int nbRead, nbWrite;
  char *idClient;
  int verbose = 0;

  if (argc != 2 && argc != 3) {
    printf("Usage = client [-v] identifiantClient\n");
    exit(EXIT_FAILURE);
  }

  idClient = argv[argc-1];
  if (argc == 3) {
    verbose = 1;
  };

  /* Prparation du champ pointAcces de la requte */
  sprintf(msgConnexion.infoSup.pointAcces, "./serveur-client.%d", getpid());

  /* Cration du tube nomm du client */
  if (mkfifo(msgConnexion.infoSup.pointAcces, S_IRUSR|S_IWUSR) < 0) {
    if (errno != EEXIST)
      error_at_line(EXIT_FAILURE, errno, __FILE__, __LINE__, "mkfifo(%s)", msgConnexion.infoSup.pointAcces);
    else
      error_at_line(0, errno, __FILE__, __LINE__, "mkfifo(%s) ==> ce tube existe deja\nOn suppose que c'est un pipe nomme et qu'on peut continuer le programme sans probleme", msgConnexion.infoSup.pointAcces);
  }

  /* Envoi de la requte vers le serveur */
  printf("Client %6s : Connexion au serveur...\n", idClient);
  fdW = open(NOM_POINT_ACCES_SERV, O_WRONLY);
  if (fdW == -1)
    error_at_line(EXIT_FAILURE, errno, __FILE__, __LINE__, "open(%s)", NOM_POINT_ACCES_SERV);
  nbWrite = write(fdW, &msgConnexion, sizeof(msgConnexion));
  if (nbWrite < sizeof(msgConnexion))
    error_at_line(EXIT_FAILURE, errno, __FILE__, __LINE__, "pb write sur tube nomme");

  /* On n'aura pas d'autres messages  envoyer sur ce tube. On peut donc   */
  /* le fermer                                                             */
  if (close(fdW) < 0)
    error_at_line(EXIT_FAILURE, errno, __FILE__, __LINE__, "close");

  /* On ouvre seulement maintenant le tube pour couter la rponse, sinon  */
  /* on serait bloqu jusqu' ce que le serveur ouvre le pipe en criture  */
  /* pour envoyer sa rponse. Si on ouvrait le tube avant d'envoyer la     */
  /* requte, le serveur ne recevrait jamais la requte et donc n'ouvrirait*/
  /* jamais le tube ==> On serait bloqu indfiniment.                     */
  fdR = open(msgConnexion.infoSup.pointAcces, O_RDONLY);
  if (fdR == -1)
    error_at_line(EXIT_FAILURE, errno, __FILE__, __LINE__, "open(%s)", msgConnexion.infoSup.pointAcces);

  /* On lit la rponse */
  nbRead = read(fdR, &msgRepConnexion, sizeof(msgRepConnexion));
  if (nbRead != sizeof(msgRepConnexion))
    error_at_line(EXIT_FAILURE, errno, __FILE__, __LINE__, "Communication avec le serveur probablement rompue\n");

  /* On analyse la rponse */
  if (msgRepConnexion.typ != CONNEXION_OK)
    error_at_line(EXIT_FAILURE, 0, __FILE__, __LINE__, "Le serveur m'a renvoye un type de message inconnu : %d", msgRepConnexion.typ);

  printf("Client %6s : Connecte !\n", idClient);

  /* On demande au serveur les differents M. et Mme */
  printf("Client %6s : Recuperation liste des M. et Mme...\n", idClient);
  fdW = open(msgRepConnexion.infoSup.pointAcces, O_WRONLY);
  if (fdW == -1)
    error_at_line(EXIT_FAILURE, errno, __FILE__, __LINE__, "open(%s)", msgRepConnexion.infoSup.pointAcces);

  do {
    nbWrite = write(fdW, &msgRequete, sizeof(msgRequete));
    if (nbWrite < sizeof(msgRequete))
      error_at_line(EXIT_FAILURE, errno, __FILE__, __LINE__, "pb write sur tube nomme");
    nbRead = read(fdR, &msgRepRequete, sizeof(msgRepRequete));
    if (nbRead != sizeof(msgRepRequete))
      error_at_line(EXIT_FAILURE, errno, __FILE__, __LINE__, "Communication avec le serveur probablement rompue\n");
    if (verbose && (msgRepRequete.typ == REQUETE_OK)) {
      printf("Client %6s : Nouveau M. et Mme = %s", idClient, msgRepRequete.infoSup.mEtMme);
    }
  } while (msgRepRequete.typ == REQUETE_OK);
  printf("Client %6s : Recuperation terminee !\n", idClient);

  /* On n'aura pas d'autres messages  envoyer sur ce tube. On peut donc   */
  /* le fermer                                                             */
  if (close(fdW) < 0)
    error_at_line(EXIT_FAILURE, errno, __FILE__, __LINE__, "close");

  /* On ferme le tube ct rception */
  if (close(fdR) < 0)
    error_at_line(EXIT_FAILURE, errno, __FILE__, __LINE__, "close");

  /* On dtruit le tube nomm du client (puisque ce tube nomm ne servira */
  /* plus jamais.                                                         */
  if (unlink(msgConnexion.infoSup.pointAcces) < 0)
    error_at_line(EXIT_FAILURE, errno, __FILE__, __LINE__, "unlink");

  return EXIT_SUCCESS;
}


   
