#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdio.h>
#include <ctype.h>
#include <error.h>
#include <errno.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>

#include "client-serveur.h"

static int rangClient = 0;

void traiterConnexion(message_t msg){
  message_t reponse, requete;
  FILE *f;
  char nomTube[TAILLE_POINT_ACCES];
  char mEtMme[TAILLE_M_ET_MME];
  int fdW, fdR;
  int nbWrite;

  rangClient++;
  printf("%6d-ieme client a traiter (point d'access client = %s)\n", rangClient, msg.infoSup.pointAcces);

  /* On vrifie qu'on peut ouvrir le fichier pour les requtes ultrieures. */
  f = fopen("mEtMme.txt", "r");
  if (f == NULL)
    error_at_line(EXIT_FAILURE, errno, __FILE__, __LINE__, "pas pu ouvrir mEtMme.txt");

  /* On ouvre le tube nomm sur lequel le client pourra venir nous faire */
  /* les demandes ultrieures.                                           */
  sprintf(nomTube, "%s.%d", NOM_POINT_ACCES_SERV,rangClient);
  if (mkfifo(nomTube, S_IRUSR|S_IWUSR) < 0) {
    if (errno != EEXIST)
      error_at_line(EXIT_FAILURE, errno, __FILE__, __LINE__, "mkfifo pour requetes client-->serveur");
   else
      error_at_line(EXIT_FAILURE, errno, __FILE__, __LINE__, "Tube nomme %s existe deja, ce qui ne devrait jamais arriver puisqu'on cree un tube dedie pour chaque client ==> le serveur s'arrete", nomTube);
  }

  /* Connexion au point d'accs client et rponse */
  fdW = open(msg.infoSup.pointAcces, O_WRONLY);
  if (fdW == -1)
    error_at_line(EXIT_FAILURE, errno, __FILE__, __LINE__, "Impossible de se connecter au tube %s", msg.infoSup.pointAcces);
  reponse.typ = CONNEXION_OK;
  strncpy(reponse.infoSup.pointAcces, nomTube, TAILLE_POINT_ACCES);
  reponse.infoSup.pointAcces[TAILLE_POINT_ACCES-1]='\0';
  nbWrite = write(fdW, &reponse, sizeof(reponse));
  if (nbWrite < sizeof(reponse))
    error_at_line(EXIT_FAILURE, errno, __FILE__, __LINE__, "Pb ecriture sur tube nomme %s", msg.infoSup.pointAcces);

  /* Ouverture de ce tube nomm */
  fdR = open(nomTube, O_RDONLY);
  if (fdR == -1)
    error_at_line(EXIT_FAILURE, errno, __FILE__, __LINE__, "open tube pour requetes client-->serveur");

  /* Attente et traitement d'une requete du client */
  do{
    int nbRead = read(fdR, &requete, sizeof(requete));
    if (nbRead != sizeof(requete)) {
      /* Le client a ferme le tube : on sort */
      break;
    }
    /* On traite la requte */
    if ((feof(f) == 0) && (fgets(mEtMme, sizeof(mEtMme), f) != NULL)) {
      reponse.typ = REQUETE_OK;
      strncpy(reponse.infoSup.mEtMme, mEtMme, TAILLE_M_ET_MME);
    } else {
      reponse.typ = REQUETE_KO;
    }
    nbWrite = write(fdW, &reponse, sizeof(reponse));
    if (nbWrite < sizeof(reponse))
      error_at_line(EXIT_FAILURE, errno, __FILE__, __LINE__, "Pb ecriture sur tube nomme %s", msg.infoSup.pointAcces);
  }while(1);

  /* On ferme tout. */
  if (fclose(f) < 0)
    error_at_line(EXIT_FAILURE, errno, __FILE__, __LINE__, "fclose");
  if (close(fdR) < 0)
    error_at_line(EXIT_FAILURE, errno, __FILE__, __LINE__, "close");
  if (close(fdW) < 0)
    error_at_line(EXIT_FAILURE, errno, __FILE__, __LINE__, "close");

  /* Ce tube ne sera plus utilis : on peut le supprimer. */
  if (unlink(nomTube) < 0)
    error_at_line(EXIT_FAILURE, errno, __FILE__, __LINE__, "unlink");
}

int main() {
  message_t msg;
  int fdR;
  int nbRead;

  /* Cration du tube nomm du serveur */
  if (mkfifo(NOM_POINT_ACCES_SERV, S_IRUSR|S_IWUSR) < 0) {
    if (errno != EEXIST)
      error_at_line(EXIT_FAILURE, errno, __FILE__, __LINE__, "mkfifo(%s)", 
		    NOM_POINT_ACCES_SERV);
    else
      error_at_line(0, errno, __FILE__, __LINE__, "mkfifo(%s) ==> ce tube existe deja\nOn suppose que c'est un pipe nomme et qu'on peut continuer le programme sans probleme",
		    NOM_POINT_ACCES_SERV);
  }

  /* Ouverture de ce tube nomm (equivalent a une ouverture de connexion).
     NB : On l'ouvre en RDWR et pas en RDONLY,
     car (cf. Blaess) : si nous ouvrons le client en lecture seule,  chaque
     fois que le client se termine, la lecture du tube nomme va echouer
     car le noyau detecte une fermeture de fichier. En demandant un tube
     en lecture et en ecriture, nous evitons ce genre de situation, car il
     reste toujours au moins un descripteur ouvert sur la sortie
  */
  fdR = open(NOM_POINT_ACCES_SERV, O_RDWR);
  if (fdR == -1)
    error_at_line(EXIT_FAILURE, errno, __FILE__, __LINE__, "open(%s)",
		  NOM_POINT_ACCES_SERV);

  /* Attente de requtes */
  do{
    nbRead = read(fdR, &msg, sizeof(msg));
    if (nbRead != sizeof(msg))
      error_at_line(EXIT_FAILURE, errno, __FILE__, __LINE__,
		    "read sur tube nomme %s KO",
		    NOM_POINT_ACCES_SERV);

    /* On traite la requte */
    traiterConnexion(msg);
  }while(1);
}


   
